'use strict'

var url = require('url')
var path = require('path')
var cp = require('child_process')
var temp = require('temp').track()
var semver = require('semver')
var shellescape = require('any-shell-escape')
var nugget = require('nugget')
var versionCache = require('./cache.json')
var S = require('string')
var isString = require('is-string')
var isFunction = require('lodash.isfunction')
var isFile = require('is-file')

module.exports = {
  // Returns the specific latest version for the given range from the cache as a string
  getLatestVersion: function (range) {
    var versionRange
    if (isString(range)) {
      // we ignore the -f part of the package version range so that it's formatted as a semvar
      versionRange = range.replace(/.(\d+)f(\d+)/, '.$1')
    } else if (range === undefined) {
      versionRange = ''
    } else {
      throw new Error('The version range must be a string specifying acceptable version bounds i.e. \'^2017\'')
    }
    return semver.maxSatisfying(Object.keys(versionCache), versionRange)
  },
  // Download the specified version to a temporary directory and call the callback when complete with the file name
  downloadInstaller: function (version, onDone) {
    if (!isString(version) || S(version).isEmpty()) {
      throw new Error('The version of Unity to download must be a string.')
    }
    // retrieve the download link, confirm cache format is correct
    var link = versionCache[version]
    if (!isString(link) || S(link).isEmpty()) {
      throw new Error('The specified version does not exist in the cache.')
    }
    // configure destination path
    var fileName = path.basename(url.parse(link).pathname)
    var tmpDir = temp.mkdirSync('unity-installer')
    var fullName = path.resolve(tmpDir, fileName)
    // initialise the download using the nugget tool (wget clone)
    var isInCI = process.env.CI !== undefined
    nugget([link], { dir: tmpDir, quiet: isInCI }, function () {
      isFunction(onDone) && onDone(fullName)
    })
  },
  // Invoke the downloaded Unity installer at the location specified by fileName
  invokeInstaller: function (fileName, onDone) {
    if (!isString(fileName) || S(fileName).isEmpty()) {
      throw new Error('fileName is not valid.')
    }
    if (!isFile(fileName)) {
      throw new Error('The file specified by fileName does not exist.')
    }
    var ext = path.extname(fileName)
    if (ext === '.pkg') {
      // newer Unity distributions use .pkg
      var args = ['sudo', 'installer', '-dumplog', '-target', '/', '-package', fileName]
      cp.exec(shellescape(args), { stdio: 'inherit' }, function (error) {
        isFunction(onDone) && onDone(!error)
      })
    } else if (ext === '.dmg') {
      // older Unity distributions use .dmg
      throw new Error('Installing Unity from a .dmg file is currently not supported.')
    } else {
      // we don't know what to do with this sort of file
      throw new Error('Unsupported file type: ' + ext)
    }
  }
}
