'use strict'

/* global describe, it, before, after */
var path = require('path')
var fs = require('fs')
var cp = require('child_process')
var temp = require('temp').track()
var expect = require('chai').expect
var mockery = require('mockery')
var sinon = require('sinon')

// test cache data
var testCacheData = {
  // smoke test
  '2017.1.1': 'https://netstorage.unity3d.com/unity/5d30cf096e79/MacEditorInstaller/Unity-2017.1.1f1.pkg',
  '2017.1.0': 'https://download.unity3d.com/download_unity/472613c02cf7/MacEditorInstaller/Unity-2017.1.0f3.pkg',
  '5.6.3': 'https://download.unity3d.com/download_unity/d3101c3b8468/MacEditorInstaller/Unity-5.6.3f1.pkg',
  '5.6.2': 'https://download.unity3d.com/download_unity/a2913c821e27/MacEditorInstaller/Unity-5.6.2f1.pkg',
  '5.0.0': 'https://download.unity3d.com/download_unity/5b98b70ebeb9/MacEditorInstaller/Unity-5.0.0f4.pkg',
  '4.7.2': 'https://download.unity3d.com/download_unity/unity-4.7.2.dmg',
  '4.7.1': 'https://download.unity3d.com/download_unity/unity-4.7.1.dmg',
  '4.0.0': 'https://download.unity3d.com/download_unity/unity-4.0.0.dmg',
  '3.5.7': 'https://download.unity3d.com/download_unity/unity-3.5.7.dmg',
  '3.5.6': 'https://download.unity3d.com/download_unity/unity-3.5.6.dmg',
  '3.4.0': 'https://download.unity3d.com/download_unity/unity-3.4.0.dmg',
  // test entry (make sure mocking is working)
  '1.7.4': 'https://raw.githubusercontent.com/zettaforge/unity-installer/master/README.md',
  // problematic cache entries
  '1.7.3': '',
  '1.7.2': null,
  '1.7.1': undefined,
  '1.7.0': {}
}

describe('unity-installer', function () {
  var installer
  before(function () {
    mockery.enable({
      warnOnReplace: false,
      warnOnUnregistered: false,
      useCleanCache: true
    })
    mockery.registerMock('./cache.json', testCacheData)
    installer = require('../lib/index')
  })
  after(function () {
    mockery.disable()
  })
  describe('getLatestVersion', function () {
    it('should export getLatestVersion(range)', function () {
      expect(installer.getLatestVersion).to.be.a('function')
    })
    it('should throw an error if the version range is of invalid type', function () {
      expect(installer.getLatestVersion.bind(null, 0)).to.throw()
      expect(installer.getLatestVersion.bind(null, 1)).to.throw()
      expect(installer.getLatestVersion.bind(null, 999999)).to.throw()
      expect(installer.getLatestVersion.bind(null, null)).to.throw()
      expect(installer.getLatestVersion.bind(null, {})).to.throw()
      expect(installer.getLatestVersion.bind(null, undefined)).to.not.throw()
    })
    it('should return the latest version if the version range is blank', function () {
      var latest = installer.getLatestVersion('*')
      expect(installer.getLatestVersion()).to.be.equal(latest)
    })
    it('should pass a range of smoke tests for explicit versions', function () {
      expect(installer.getLatestVersion('3.5.6')).to.be.equal('3.5.6')
      expect(installer.getLatestVersion('4.7.1')).to.be.equal('4.7.1')
      expect(installer.getLatestVersion('5.6.2')).to.be.equal('5.6.2')
      expect(installer.getLatestVersion('2017.1.0')).to.be.equal('2017.1.0')
    })
    it('should pass a range of smoke tests for version ranges using older Unity distributions', function () {
      expect(installer.getLatestVersion('>=3 <4')).to.be.equal('3.5.7')
      expect(installer.getLatestVersion('^3')).to.be.equal('3.5.7')
      expect(installer.getLatestVersion('>=4 <5')).to.be.equal('4.7.2')
      expect(installer.getLatestVersion('^4')).to.be.equal('4.7.2')
      expect(installer.getLatestVersion('>=5 <6')).to.be.equal('5.6.3')
      expect(installer.getLatestVersion('^5')).to.be.equal('5.6.3')
      expect(installer.getLatestVersion('<2017')).to.be.equal('5.6.3')
    })
  })
  describe('downloadInstaller', function () {
    it('should export downloadInstaller(version)', function () {
      expect(installer.downloadInstaller).to.be.a('function')
    })
    it('should throw if the version specified is of invalid type', function () {
      expect(installer.downloadInstaller.bind(null, 0)).to.throw()
      expect(installer.downloadInstaller.bind(null, 1)).to.throw()
      expect(installer.downloadInstaller.bind(null, 999999)).to.throw()
      expect(installer.downloadInstaller.bind(null, undefined)).to.throw()
      expect(installer.downloadInstaller.bind(null, null)).to.throw()
      expect(installer.downloadInstaller.bind(null, {})).to.throw()
    })
    it('should throw an error if the version specified is not an exact version', function () {
      expect(installer.downloadInstaller.bind(null, '*')).to.throw()
      expect(installer.downloadInstaller.bind(null, '3')).to.throw()
      expect(installer.downloadInstaller.bind(null, '4')).to.throw()
      expect(installer.downloadInstaller.bind(null, '5')).to.throw()
      expect(installer.downloadInstaller.bind(null, '5.6')).to.throw()
      expect(installer.downloadInstaller.bind(null, '2017')).to.throw()
      expect(installer.downloadInstaller.bind(null, '2017.1')).to.throw()
    })
    it('should throw an error if the version specified does not exist in the cache', function () {
      expect(installer.downloadInstaller.bind(null, '0.0.0')).to.throw()
      expect(installer.downloadInstaller.bind(null, '5.7.1')).to.throw()
    })
    it('should throw if the version specified is corrupt or entered into the cache incorrectly', function () {
      expect(installer.downloadInstaller.bind(null, '1.7.0')).to.throw()
      expect(installer.downloadInstaller.bind(null, '1.7.1')).to.throw()
      expect(installer.downloadInstaller.bind(null, '1.7.2')).to.throw()
      expect(installer.downloadInstaller.bind(null, '1.7.3')).to.throw()
    })
    it('should pass a simple smoke test for downloading a small file specified by the version cache', function () {
      this.timeout(15000)
      return new Promise(function (resolve, reject) {
        installer.downloadInstaller('1.7.4', function (fileName) {
          expect(fileName).not.to.equal('README.md')
          expect(path.basename(fileName)).to.equal('README.md')
          resolve(fileName)
        })
      })
    })
  })
  describe('invokeInstaller', function () {
    it('should export invokeInstaller(fileName)', function () {
      expect(installer.invokeInstaller).to.be.a('function')
    })
    it('should throw if the fileName specified is of invalid type', function () {
      expect(installer.invokeInstaller.bind(null, 0)).to.throw()
      expect(installer.invokeInstaller.bind(null, 1)).to.throw()
      expect(installer.invokeInstaller.bind(null, 999999)).to.throw()
      expect(installer.invokeInstaller.bind(null, undefined)).to.throw()
      expect(installer.invokeInstaller.bind(null, null)).to.throw()
      expect(installer.invokeInstaller.bind(null, {})).to.throw()
    })
    it('should throw if the file specified by fileName doesn\'t exist', function () {
      var tmpDir = temp.mkdirSync('unity-installer-tests')
      expect(installer.invokeInstaller.bind(null, path.resolve(tmpDir, 'this_file_doesnt_exist.pkg'))).to.throw()
    })
    it('should throw if the type of the file specified is neither .pkg or .dmg', function () {
      var tmpDir = temp.mkdirSync('unity-installer-tests')
      var fileName = path.resolve(tmpDir, 'Some_Installer.msi')
      fs.writeFileSync(fileName, '###')
      expect(installer.invokeInstaller.bind(null, fileName)).to.throw()
    })
    it('should throw when given a .dmg volume because they are currently not supported', function () {
      var tmpDir = temp.mkdirSync('unity-installer-tests')
      var fileName = path.resolve(tmpDir, 'Unity_installer.dmg')
      fs.writeFileSync(fileName, '###')
      expect(installer.invokeInstaller.bind(null, fileName)).to.throw()
    })
    it('should invoke the installer correctly for .pkg distributions', function () {
      var tmpDir = temp.mkdirSync('unity-installer-tests')
      var fileName = path.resolve(tmpDir, 'Unity_installer.pkg')
      fs.writeFileSync(fileName, '###')
      // hook exec to look at the line that is sent to the shell, but because we don't actually
      // invoke any I/O, there is no need for a Promise here
      sinon.stub(cp, 'exec')
      var onDoneSpy = sinon.spy()
      installer.invokeInstaller(fileName, onDoneSpy)
      expect(cp.exec.calledOnce).to.be.equal(true)
      expect(cp.exec.firstCall.args[0]).to.be.equal('sudo installer -dumplog -target / -package \'' + fileName + '\'')
      // check onDone is actually called via closure when it's included in the parameter list
      cp.exec.firstCall.args[2]()
      expect(onDoneSpy.calledOnce).to.be.equal(true)
      cp.exec.restore()
    })
  })
})
